C$Procedure      TXTBUF (Text buffer)

      SUBROUTINE TXTBUF (FILE, PNTER, LINE, BERR, LINNUM)

C$ Abstract
C
C     Buffer a text file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C     TEXT
C
C$ Declarations
      
      CHARACTER*(*)         FILE
      INTEGER               PNTER
      CHARACTER*(*)         LINE
      LOGICAL               BERR
      INTEGER               LINNUM

C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C     UNIT
C     PNTER
C     LINE
C     BERR
C     LINNUM
C      
C$ Detailed_Input
C
C
C$ Detailed_Output
C
C
C$ Parameters
C
C
C$ Exceptions
C
C     3)
C
C$ Files
C
C     None.
C
C$ Particulars
C
C
C
C$ Examples
C
C
C
C$ Restrictions
C
C
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    SPICELIB Version 1.1.0, 18-JUN-1999 (WLT)
C
C        Repaired errors in CHKIN/CHKOUT
C
C-    Beta Version 1.0.0, 19-NOV-1992 (MJS) 
C
C-&

C
C     SPICELIB functions
C
      LOGICAL               RETURN

C
C     Local parameters
C
      INTEGER               LINLEN
      PARAMETER           ( LINLEN = 132 )
      
      INTEGER               BUFSIZ
      PARAMETER           ( BUFSIZ = 2000 )
      
      INTEGER               FILEN
      PARAMETER           ( FILEN = 128 )
      
C
C     Local variables
C
      CHARACTER*(LINLEN)    BUFFER (BUFSIZ)
      CHARACTER*(FILEN)     FILNAM
      
      INTEGER               UNIT
      INTEGER               IOSTAT
      INTEGER               STACK
      INTEGER               ALTSTK
      INTEGER               BF     ( 2 )
      INTEGER               EF     ( 2 )
      INTEGER               BS     ( 2 )
      INTEGER               ER     ( 2 )
      INTEGER               HIGH
      INTEGER               LOW
      INTEGER               LINPTR
      INTEGER               TMP
      
      LOGICAL               OPENED
      LOGICAL               EXIST
      LOGICAL               EOF    ( 2 )
      

C
C     Saved variables
C
      SAVE


C
C     Standard SPICE error handling.
C
      IF ( RETURN () ) THEN
         RETURN
      ELSE
         CALL CHKIN ( 'TXTBUF' )
      END IF

C
C     This routine should never be called. If this routine is called,
C     an error is signalled.
C
      CALL SETMSG ('TXTBUF: You have called an entry which performs ' //
     .             'performs no run-time function. This may '         //
     .             'indicate a bug.'                                   )
 
      CALL SIGERR ('SPICE(BOGUSENTRY)')
 
      CALL CHKOUT ('TXTBUF')
      RETURN



C$Procedure TXTRFL (Text, read first line)

      ENTRY TXTRFL (FILE, LINE)

C$ Abstract
C
C     Read the first line of a text file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C     TEXT
C
C$ Declarations
C
C     INTEGER               UNIT
C     CHARACTER*(*)         LINE
C     LOGICAL               BERR
C     INTEGER               LINNUM
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C
C$ Detailed_Input
C
C
C$ Detailed_Output
C
C
C$ Parameters
C
C
C
C$ Exceptions
C
C     3)
C
C$ Files
C
C     None.
C
C$ Particulars
C
C
C
C$ Examples
C
C
C
C$ Restrictions
C
C
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 19-NOV-1992 (MJS) 
C
C-&

   
      IF (RETURN()) THEN
         RETURN
      ELSE
         CALL CHKIN ('TXTRFL')
      END IF
      
C     
C     Open the file. Close it first if it's already open.
C     
      FILNAM = FILE
      
      INQUIRE (FILE   = FILNAM,
     .         EXIST  = EXIST,
     .         OPENED = OPENED,
     .         NUMBER = UNIT,
     .         IOSTAT = IOSTAT)
     
      IF (IOSTAT .NE. 0) THEN
         CALL SETMSG ('Inquire on file # failed.')
         CALL ERRCH  ('#', FILNAM                )
         CALL SIGERR ('SPICE(INQUIREFAILED)'     )
         CALL CHKOUT ('TXTRFL'                   )
         RETURN
      END IF
      
      IF (EXIST .AND. OPENED) THEN
         CLOSE (UNIT)
      END IF
      
C     
C     Open the file.
C     
      CALL TXTOPR (FILNAM, UNIT)      

C
C     Set up the nominal begin and end stack elements.
C   
      ER(1) = BUFSIZ / 2
      ER(2) = ER(1)
      
      BS(1) = 1
      BS(2) = BS(1) + ER(1)
      
C     
C     Initially, we're in the first stack.
C     
      STACK  = 1
      ALTSTK = 2
      
C     
C     Fill both stacks, if we can.
C     
      CALL FILBUF (UNIT, BS(STACK), ER(STACK), BUFFER, EOF(STACK))
      
      IF (ER(STACK) .EQ. 0) THEN
         CALL SETMSG ('File # has no data.')
         CALL ERRCH  ('#', FILNAM          )
         CALL SIGERR ('SPICE(NODATA)'      )
         CALL CHKOUT ('TXTRFL'             )
         RETURN
      END IF
      
      BF(STACK) = 1
      EF(STACK) = ER(STACK)
       
      IF (.NOT. EOF(STACK)) THEN
         
         CALL FILBUF (UNIT, BS(ALTSTK), ER(ALTSTK), BUFFER, EOF(ALTSTK))
         
         IF (ER(ALTSTK) .EQ. 0) THEN
            
            EOF(STACK) = .TRUE.
            LOW         = STACK
            HIGH        = LOW
         
         ELSE
         
            BF(ALTSTK) = EF(STACK)  + 1
            EF(ALTSTK) = BF(ALTSTK) + ER(ALTSTK) - 1
            LOW        = STACK
            HIGH       = ALTSTK
      
         END IF
         
      ELSE
         
         LOW  = STACK
         HIGH = LOW
         
      END IF
      
      LINPTR = 1
      LINE   = BUFFER(1)
         
      CALL CHKOUT ('TXTRFL')   
      RETURN



C$Procedure TXTRRL (Text, read relative line)

      ENTRY TXTRRL (PNTER, LINE, BERR, LINNUM)

C$ Abstract
C
C     Read the first line of a text file.
C
C$ Required_Reading
C
C     None.
C
C$ Keywords
C
C     FILES
C     TEXT
C
C$ Declarations
C
C     INTEGER               PNTER
C     CHARACTER*(*)         LINE
C     LOGICAL               BERR
C     INTEGER               LINNUM
C
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C
C$ Detailed_Input
C
C
C$ Detailed_Output
C
C
C$ Parameters
C
C
C
C$ Exceptions
C
C     3)
C
C$ Files
C
C     None.
C
C$ Particulars
C
C
C
C$ Examples
C
C
C
C$ Restrictions
C
C
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     M.J. Spencer   (JPL)
C
C$ Version
C
C-    Beta Version 1.0.0, 19-NOV-1992 (MJS) 
C
C-&

      IF (RETURN()) THEN
         RETURN
      ELSE
         CALL CHKIN ('TXTRRL')
      END IF

C     
C     Where are we now?
C     
      LINPTR = LINPTR + PNTER
      
C     
C     Handle the trivial case first: LINPTR is in STACK.
C     
      IF (LINPTR .GE. BF(STACK) .AND. 
     .    LINPTR .LE. EF(STACK)       ) THEN
     
         LINNUM = LINPTR
         LINE   = BUFFER(BS(STACK)+LINPTR-BF(STACK))
         BERR   = .FALSE.
         
         CALL CHKOUT ('TXTRRL')
         RETURN
         
      END IF
      
C     
C     Handle the case where we need to jump back to the LOW stack.
C     
      IF (LINPTR .LT. BF(STACK)) THEN
         
         IF (STACK .EQ. LOW) THEN
            
            LINPTR = BF(STACK)
            BERR   = .TRUE.
            
         ELSE
         
            STACK  = ALTSTK
            ALTSTK = HIGH
            
            IF (LINPTR .LT. BF(STACK)) THEN
               
               LINPTR = BF(STACK)
               BERR   = .TRUE.
               
            ELSE 
            
               BERR   = .FALSE.
               
            END IF
            
         END IF
         
         LINNUM = LINPTR
         LINE   = BUFFER(BS(STACK)+LINPTR-BF(STACK))
         
         CALL CHKOUT ('TXTRRL')
         RETURN
      
      END IF

C     
C     Well, now we now it's somewhere ahead. Make sure STACK is HIGH.
C                 
      IF (STACK .NE. HIGH) THEN
         
         TMP    = STACK
         STACK  = ALTSTK
         ALTSTK = TMP
      
      END IF
      
      DO WHILE (LINPTR .GT. EF(STACK) .AND. .NOT. EOF(STACK))
         
         LOW    = STACK
         STACK  = ALTSTK
         ALTSTK = LOW
         HIGH   = STACK
         
         CALL FILBUF (UNIT, BS(STACK), ER(STACK), BUFFER, EOF(STACK))
            
         IF (ER(STACK) .EQ. 0) THEN
               
            STACK      = ALTSTK
            EOF(STACK) = .TRUE.
            HIGH       = STACK
            
         ELSE
            
            BF(STACK) = EF(ALTSTK) + 1
            EF(STACK) = BF(STACK)  + ER(STACK) - 1
               
         END IF
               
      END DO
      
      BERR   = LINPTR .GT. EF(STACK)
      LINPTR = MIN (LINPTR, EF(STACK))
      
      LINNUM = LINPTR
      LINE   = BUFFER(BS(STACK)+LINPTR-BF(STACK))
      
      CALL CHKOUT ('TXTRRL')
      RETURN
      END

